@@ -1,7 +1,7 @@
 # This module provides interpolation for functions defined on a grid.
 #
 # Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
-# last revision: 2008-8-18
+# last revision: 2011-2-18
 #
 
 """
@@ -35,7 +35,7 @@ class InterpolatingFunction:
         """
         @param axes: a sequence of one-dimensional arrays, one for each
             variable, specifying the values of the variables at
-            the grid points
+            the grid points in ascending order
         @type axes: sequence of N.array
 
         @param values: the function values on the grid
@@ -53,6 +53,14 @@ class InterpolatingFunction:
         """
         if len(axes) > len(values.shape):
             raise ValueError('Inconsistent arguments')
+        for i, axis in enumerate(axes):
+            if len(axis.shape) != 1:
+                raise ValueError("Axes must be 1D arrays")
+            if len(axis) != values.shape[i]:
+                raise ValueError("Axes must match value array")
+            if N.logical_or.reduce(axis[1:]-axis[:-1] <= 0.):
+                raise ValueError("Axis values must be distinct and "
+                                 "in ascending order")
         self.axes = list(axes)
         self.shape = sum([axis.shape for axis in self.axes], ())
         self.values = values
@@ -404,8 +412,13 @@ class NetCDFInterpolatingFunction(InterpolatingFunction):
         """
         from Scientific.IO.NetCDF import NetCDFFile
         self.file = NetCDFFile(filename, 'r')
-        self.axes = map(lambda n, f=self.file: f.variables[n], axesnames)
+        self.axes = [self.file.variables[n] for n in axesnames]
+        for a in self.axes:
+            if len(a.dimensions) != 1:
+                raise ValueError("axes must be 1d arrays")
         self.values = self.file.variables[variablename]
+        if tuple(v.dimensions[0] for v in self.axes) != self.values.dimensions:
+            raise ValueError("axes and values have incompatible dimensions")
         self.default = default
         self.shape = ()
         for axis in self.axes:
@@ -426,7 +439,7 @@ NetCDFInterpolatingFunction._constructor = InterpolatingFunction
 
 def _lookup(point, axis, period):
     if period is None:
-        j = N.int_sum(N.less_equal(axis, point))
+        j = int(N.int_sum(N.less_equal(axis, point)))
         if j == len(axis):
             if N.fabs(point - axis[j-1]) < 1.e-9:
                 return index_expression[j-2:j:1], 1.
@@ -439,7 +452,7 @@ def _lookup(point, axis, period):
         return index_expression[i:j+1:1], weight
     else:
         point = axis[0] + (point-axis[0]) % period
-        j = N.int_sum(N.less_equal(axis, point))
+        j = int(N.int_sum(N.less_equal(axis, point)))
         i = j-1
         if j == len(axis):
             weight = (point-axis[i])/(axis[0]+period-axis[i])
